from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.cost_config_type import CostConfigType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.time_range_config import TimeRangeConfig


T = TypeVar("T", bound="CostConfigResponse")


@_attrs_define
class CostConfigResponse:
    """Response model for cost configuration

    Attributes:
        id (str):
        config_type (CostConfigType): Types of cost configurations
        value (float):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        created_by (str):
        updated_by (str):
        description (None | str | Unset):
        time_ranges (None | TimeRangeConfig | Unset):
    """

    id: str
    config_type: CostConfigType
    value: float
    created_at: datetime.datetime
    updated_at: datetime.datetime
    created_by: str
    updated_by: str
    description: None | str | Unset = UNSET
    time_ranges: None | TimeRangeConfig | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.time_range_config import TimeRangeConfig

        id = self.id

        config_type = self.config_type.value

        value = self.value

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        created_by = self.created_by

        updated_by = self.updated_by

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        time_ranges: dict[str, Any] | None | Unset
        if isinstance(self.time_ranges, Unset):
            time_ranges = UNSET
        elif isinstance(self.time_ranges, TimeRangeConfig):
            time_ranges = self.time_ranges.to_dict()
        else:
            time_ranges = self.time_ranges

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "config_type": config_type,
                "value": value,
                "created_at": created_at,
                "updated_at": updated_at,
                "created_by": created_by,
                "updated_by": updated_by,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if time_ranges is not UNSET:
            field_dict["time_ranges"] = time_ranges

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.time_range_config import TimeRangeConfig

        d = dict(src_dict)
        id = d.pop("id")

        config_type = CostConfigType(d.pop("config_type"))

        value = d.pop("value")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        created_by = d.pop("created_by")

        updated_by = d.pop("updated_by")

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_time_ranges(data: object) -> None | TimeRangeConfig | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                time_ranges_type_0 = TimeRangeConfig.from_dict(data)

                return time_ranges_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TimeRangeConfig | Unset, data)

        time_ranges = _parse_time_ranges(d.pop("time_ranges", UNSET))

        cost_config_response = cls(
            id=id,
            config_type=config_type,
            value=value,
            created_at=created_at,
            updated_at=updated_at,
            created_by=created_by,
            updated_by=updated_by,
            description=description,
            time_ranges=time_ranges,
        )

        cost_config_response.additional_properties = d
        return cost_config_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.app_types import AppTypes
from ..types import UNSET, Unset

T = TypeVar("T", bound="AppCredsRead")


@_attrs_define
class AppCredsRead:
    """
    Attributes:
        id (None | str | Unset):
        app_name (AppTypes | None | Unset):
    """

    id: None | str | Unset = UNSET
    app_name: AppTypes | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id: None | str | Unset
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        app_name: None | str | Unset
        if isinstance(self.app_name, Unset):
            app_name = UNSET
        elif isinstance(self.app_name, AppTypes):
            app_name = self.app_name.value
        else:
            app_name = self.app_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if app_name is not UNSET:
            field_dict["app_name"] = app_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        id = _parse_id(d.pop("id", UNSET))

        def _parse_app_name(data: object) -> AppTypes | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                app_name_type_0 = AppTypes(data)

                return app_name_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AppTypes | None | Unset, data)

        app_name = _parse_app_name(d.pop("app_name", UNSET))

        app_creds_read = cls(
            id=id,
            app_name=app_name,
        )

        app_creds_read.additional_properties = d
        return app_creds_read

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.search_web_config_search_type import SearchWebConfigSearchType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.search_text_params import SearchTextParams


T = TypeVar("T", bound="SearchWebConfig")


@_attrs_define
class SearchWebConfig:
    """
    Attributes:
        search_params (SearchTextParams):
        search_type (SearchWebConfigSearchType | Unset):  Default: SearchWebConfigSearchType.WEB.
    """

    search_params: SearchTextParams
    search_type: SearchWebConfigSearchType | Unset = SearchWebConfigSearchType.WEB
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        search_params = self.search_params.to_dict()

        search_type: str | Unset = UNSET
        if not isinstance(self.search_type, Unset):
            search_type = self.search_type.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "search_params": search_params,
            }
        )
        if search_type is not UNSET:
            field_dict["search_type"] = search_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.search_text_params import SearchTextParams

        d = dict(src_dict)
        search_params = SearchTextParams.from_dict(d.pop("search_params"))

        _search_type = d.pop("search_type", UNSET)
        search_type: SearchWebConfigSearchType | Unset
        if isinstance(_search_type, Unset):
            search_type = UNSET
        else:
            search_type = SearchWebConfigSearchType(_search_type)

        search_web_config = cls(
            search_params=search_params,
            search_type=search_type,
        )

        search_web_config.additional_properties = d
        return search_web_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.adp_client_credentials_request import ADPClientCredentialsRequest
from ...models.http_validation_error import HTTPValidationError
from ...types import Response


def _get_kwargs(
    *,
    body: ADPClientCredentialsRequest,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/auth/adp/client-credentials",
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Any | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = response.json()
        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[Any | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    body: ADPClientCredentialsRequest,
) -> Response[Any | HTTPValidationError]:
    r""" Adp Client Credentials

     Authenticate using ADP client credentials flow with Mutual SSL.

    ADP mandates mTLS where both client and server verify each other's
    identity through SSL certificates.

    Certificate paths can be:
    - Local file paths: /path/to/cert.pem
    - S3 URLs: s3://bucket/path/cert.pem or https://bucket.s3.region.amazonaws.com/path/cert.pem

    S3 certificates will be automatically downloaded to temporary files,
    used for authentication, then securely deleted.

    Args:
        body (ADPClientCredentialsRequest): Request model for ADP client credentials
            authentication.

            ADP requires Mutual SSL (mTLS) Authentication:
            - client_id: Your ADP client ID from ADP Developer Portal
            - client_secret: Your ADP client secret
            - ssl_cert_path: Path to SSL certificate file (PEM or PFX format)
            - ssl_key_path: Path to private key file (optional if embedded in cert)

            Supported path formats for certificates:
            - Local file path: /path/to/cert.pem
            - S3 URI: s3://bucket-name/path/to/cert.pem
            - S3 HTTPS URL: https://bucket-name.s3.region.amazonaws.com/path/to/cert.pem

            Certificate Setup:
            1. Obtain SSL certificate and private key from:
               - ADP Partners: Developer Self-Service Portal (automated)
               - ADP Clients: ADP Certificate Signing Tool (manual CSR)
               - API Central Users: API Central (automated)

            2. Upload certificates to S3 (recommended) or provide local paths:
               s3://your-bucket/adp/cert.pem
               s3://your-bucket/adp/key.key

            3. Optionally combine into single PFX file:
               openssl pkcs12 -export -out adp.pfx -name "OrgName Mutual SSL" \
                   -inkey private-key.key -in public-certificate.pem

            Note: Certificates are valid for 2 years. ADP sends renewal notices
            60 days before expiration.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
     """

    kwargs = _get_kwargs(
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    body: ADPClientCredentialsRequest,
) -> Any | HTTPValidationError | None:
    r""" Adp Client Credentials

     Authenticate using ADP client credentials flow with Mutual SSL.

    ADP mandates mTLS where both client and server verify each other's
    identity through SSL certificates.

    Certificate paths can be:
    - Local file paths: /path/to/cert.pem
    - S3 URLs: s3://bucket/path/cert.pem or https://bucket.s3.region.amazonaws.com/path/cert.pem

    S3 certificates will be automatically downloaded to temporary files,
    used for authentication, then securely deleted.

    Args:
        body (ADPClientCredentialsRequest): Request model for ADP client credentials
            authentication.

            ADP requires Mutual SSL (mTLS) Authentication:
            - client_id: Your ADP client ID from ADP Developer Portal
            - client_secret: Your ADP client secret
            - ssl_cert_path: Path to SSL certificate file (PEM or PFX format)
            - ssl_key_path: Path to private key file (optional if embedded in cert)

            Supported path formats for certificates:
            - Local file path: /path/to/cert.pem
            - S3 URI: s3://bucket-name/path/to/cert.pem
            - S3 HTTPS URL: https://bucket-name.s3.region.amazonaws.com/path/to/cert.pem

            Certificate Setup:
            1. Obtain SSL certificate and private key from:
               - ADP Partners: Developer Self-Service Portal (automated)
               - ADP Clients: ADP Certificate Signing Tool (manual CSR)
               - API Central Users: API Central (automated)

            2. Upload certificates to S3 (recommended) or provide local paths:
               s3://your-bucket/adp/cert.pem
               s3://your-bucket/adp/key.key

            3. Optionally combine into single PFX file:
               openssl pkcs12 -export -out adp.pfx -name "OrgName Mutual SSL" \
                   -inkey private-key.key -in public-certificate.pem

            Note: Certificates are valid for 2 years. ADP sends renewal notices
            60 days before expiration.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
     """

    return sync_detailed(
        client=client,
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    body: ADPClientCredentialsRequest,
) -> Response[Any | HTTPValidationError]:
    r""" Adp Client Credentials

     Authenticate using ADP client credentials flow with Mutual SSL.

    ADP mandates mTLS where both client and server verify each other's
    identity through SSL certificates.

    Certificate paths can be:
    - Local file paths: /path/to/cert.pem
    - S3 URLs: s3://bucket/path/cert.pem or https://bucket.s3.region.amazonaws.com/path/cert.pem

    S3 certificates will be automatically downloaded to temporary files,
    used for authentication, then securely deleted.

    Args:
        body (ADPClientCredentialsRequest): Request model for ADP client credentials
            authentication.

            ADP requires Mutual SSL (mTLS) Authentication:
            - client_id: Your ADP client ID from ADP Developer Portal
            - client_secret: Your ADP client secret
            - ssl_cert_path: Path to SSL certificate file (PEM or PFX format)
            - ssl_key_path: Path to private key file (optional if embedded in cert)

            Supported path formats for certificates:
            - Local file path: /path/to/cert.pem
            - S3 URI: s3://bucket-name/path/to/cert.pem
            - S3 HTTPS URL: https://bucket-name.s3.region.amazonaws.com/path/to/cert.pem

            Certificate Setup:
            1. Obtain SSL certificate and private key from:
               - ADP Partners: Developer Self-Service Portal (automated)
               - ADP Clients: ADP Certificate Signing Tool (manual CSR)
               - API Central Users: API Central (automated)

            2. Upload certificates to S3 (recommended) or provide local paths:
               s3://your-bucket/adp/cert.pem
               s3://your-bucket/adp/key.key

            3. Optionally combine into single PFX file:
               openssl pkcs12 -export -out adp.pfx -name "OrgName Mutual SSL" \
                   -inkey private-key.key -in public-certificate.pem

            Note: Certificates are valid for 2 years. ADP sends renewal notices
            60 days before expiration.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Any | HTTPValidationError]
     """

    kwargs = _get_kwargs(
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    body: ADPClientCredentialsRequest,
) -> Any | HTTPValidationError | None:
    r""" Adp Client Credentials

     Authenticate using ADP client credentials flow with Mutual SSL.

    ADP mandates mTLS where both client and server verify each other's
    identity through SSL certificates.

    Certificate paths can be:
    - Local file paths: /path/to/cert.pem
    - S3 URLs: s3://bucket/path/cert.pem or https://bucket.s3.region.amazonaws.com/path/cert.pem

    S3 certificates will be automatically downloaded to temporary files,
    used for authentication, then securely deleted.

    Args:
        body (ADPClientCredentialsRequest): Request model for ADP client credentials
            authentication.

            ADP requires Mutual SSL (mTLS) Authentication:
            - client_id: Your ADP client ID from ADP Developer Portal
            - client_secret: Your ADP client secret
            - ssl_cert_path: Path to SSL certificate file (PEM or PFX format)
            - ssl_key_path: Path to private key file (optional if embedded in cert)

            Supported path formats for certificates:
            - Local file path: /path/to/cert.pem
            - S3 URI: s3://bucket-name/path/to/cert.pem
            - S3 HTTPS URL: https://bucket-name.s3.region.amazonaws.com/path/to/cert.pem

            Certificate Setup:
            1. Obtain SSL certificate and private key from:
               - ADP Partners: Developer Self-Service Portal (automated)
               - ADP Clients: ADP Certificate Signing Tool (manual CSR)
               - API Central Users: API Central (automated)

            2. Upload certificates to S3 (recommended) or provide local paths:
               s3://your-bucket/adp/cert.pem
               s3://your-bucket/adp/key.key

            3. Optionally combine into single PFX file:
               openssl pkcs12 -export -out adp.pfx -name "OrgName Mutual SSL" \
                   -inkey private-key.key -in public-certificate.pem

            Note: Certificates are valid for 2 years. ADP sends renewal notices
            60 days before expiration.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Any | HTTPValidationError
     """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ChatCompletionFunctionMessageParam")


@_attrs_define
class ChatCompletionFunctionMessageParam:
    """
    Attributes:
        content (None | str):
        name (str):
        role (Literal['function']):
    """

    content: None | str
    name: str
    role: Literal["function"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        content: None | str
        content = self.content

        name = self.name

        role = self.role

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "content": content,
                "name": name,
                "role": role,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_content(data: object) -> None | str:
            if data is None:
                return data
            return cast(None | str, data)

        content = _parse_content(d.pop("content"))

        name = d.pop("name")

        role = cast(Literal["function"], d.pop("role"))
        if role != "function":
            raise ValueError(f"role must match const 'function', got '{role}'")

        chat_completion_function_message_param = cls(
            content=content,
            name=name,
            role=role,
        )

        chat_completion_function_message_param.additional_properties = d
        return chat_completion_function_message_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PartitionParametersParamsStrategyType0(str, Enum):
    AUTO = "auto"
    FAST = "fast"
    HI_RES = "hi_res"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="NetSuiteTBARequest")


@_attrs_define
class NetSuiteTBARequest:
    """Token-Based Authentication (TBA) for NetSuite SuiteTalk (OAuth 1.0):
    - realm == account_id
    - signature method HMAC-SHA256 (used per-request)

        Attributes:
            account_id (str):
            consumer_key (str):
            consumer_secret (str):
            access_token (str):
            token_secret (str):
            base_url (None | str | Unset):
    """

    account_id: str
    consumer_key: str
    consumer_secret: str
    access_token: str
    token_secret: str
    base_url: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        account_id = self.account_id

        consumer_key = self.consumer_key

        consumer_secret = self.consumer_secret

        access_token = self.access_token

        token_secret = self.token_secret

        base_url: None | str | Unset
        if isinstance(self.base_url, Unset):
            base_url = UNSET
        else:
            base_url = self.base_url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "account_id": account_id,
                "consumer_key": consumer_key,
                "consumer_secret": consumer_secret,
                "access_token": access_token,
                "token_secret": token_secret,
            }
        )
        if base_url is not UNSET:
            field_dict["base_url"] = base_url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        account_id = d.pop("account_id")

        consumer_key = d.pop("consumer_key")

        consumer_secret = d.pop("consumer_secret")

        access_token = d.pop("access_token")

        token_secret = d.pop("token_secret")

        def _parse_base_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        base_url = _parse_base_url(d.pop("base_url", UNSET))

        net_suite_tba_request = cls(
            account_id=account_id,
            consumer_key=consumer_key,
            consumer_secret=consumer_secret,
            access_token=access_token,
            token_secret=token_secret,
            base_url=base_url,
        )

        net_suite_tba_request.additional_properties = d
        return net_suite_tba_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.region import Region
from ..models.search_text_params_safesearch import SearchTextParamsSafesearch
from ..models.timelimit import Timelimit
from ..types import UNSET, Unset

T = TypeVar("T", bound="SearchTextParams")


@_attrs_define
class SearchTextParams:
    """
    Attributes:
        keywords (str | Unset):  Default: ''.
        region (Region | Unset):
        safesearch (SearchTextParamsSafesearch | Unset):  Default: SearchTextParamsSafesearch.MODERATE.
        timelimit (None | Timelimit | Unset):
        max_results (int | Unset):  Default: 3.
    """

    keywords: str | Unset = ""
    region: Region | Unset = UNSET
    safesearch: SearchTextParamsSafesearch | Unset = SearchTextParamsSafesearch.MODERATE
    timelimit: None | Timelimit | Unset = UNSET
    max_results: int | Unset = 3
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        keywords = self.keywords

        region: str | Unset = UNSET
        if not isinstance(self.region, Unset):
            region = self.region.value

        safesearch: str | Unset = UNSET
        if not isinstance(self.safesearch, Unset):
            safesearch = self.safesearch.value

        timelimit: None | str | Unset
        if isinstance(self.timelimit, Unset):
            timelimit = UNSET
        elif isinstance(self.timelimit, Timelimit):
            timelimit = self.timelimit.value
        else:
            timelimit = self.timelimit

        max_results = self.max_results

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if keywords is not UNSET:
            field_dict["keywords"] = keywords
        if region is not UNSET:
            field_dict["region"] = region
        if safesearch is not UNSET:
            field_dict["safesearch"] = safesearch
        if timelimit is not UNSET:
            field_dict["timelimit"] = timelimit
        if max_results is not UNSET:
            field_dict["max_results"] = max_results

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        keywords = d.pop("keywords", UNSET)

        _region = d.pop("region", UNSET)
        region: Region | Unset
        if isinstance(_region, Unset):
            region = UNSET
        else:
            region = Region(_region)

        _safesearch = d.pop("safesearch", UNSET)
        safesearch: SearchTextParamsSafesearch | Unset
        if isinstance(_safesearch, Unset):
            safesearch = UNSET
        else:
            safesearch = SearchTextParamsSafesearch(_safesearch)

        def _parse_timelimit(data: object) -> None | Timelimit | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                timelimit_type_0 = Timelimit(data)

                return timelimit_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Timelimit | Unset, data)

        timelimit = _parse_timelimit(d.pop("timelimit", UNSET))

        max_results = d.pop("max_results", UNSET)

        search_text_params = cls(
            keywords=keywords,
            region=region,
            safesearch=safesearch,
            timelimit=timelimit,
            max_results=max_results,
        )

        search_text_params.additional_properties = d
        return search_text_params

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user import User


T = TypeVar("T", bound="Session")


@_attrs_define
class Session:
    """
    Attributes:
        access_token (str):
        refresh_token (str):
        expires_in (int):
        token_type (str):
        user (User):
        provider_token (None | str | Unset):
        provider_refresh_token (None | str | Unset):
        expires_at (int | None | Unset):
    """

    access_token: str
    refresh_token: str
    expires_in: int
    token_type: str
    user: User
    provider_token: None | str | Unset = UNSET
    provider_refresh_token: None | str | Unset = UNSET
    expires_at: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        access_token = self.access_token

        refresh_token = self.refresh_token

        expires_in = self.expires_in

        token_type = self.token_type

        user = self.user.to_dict()

        provider_token: None | str | Unset
        if isinstance(self.provider_token, Unset):
            provider_token = UNSET
        else:
            provider_token = self.provider_token

        provider_refresh_token: None | str | Unset
        if isinstance(self.provider_refresh_token, Unset):
            provider_refresh_token = UNSET
        else:
            provider_refresh_token = self.provider_refresh_token

        expires_at: int | None | Unset
        if isinstance(self.expires_at, Unset):
            expires_at = UNSET
        else:
            expires_at = self.expires_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "access_token": access_token,
                "refresh_token": refresh_token,
                "expires_in": expires_in,
                "token_type": token_type,
                "user": user,
            }
        )
        if provider_token is not UNSET:
            field_dict["provider_token"] = provider_token
        if provider_refresh_token is not UNSET:
            field_dict["provider_refresh_token"] = provider_refresh_token
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.user import User

        d = dict(src_dict)
        access_token = d.pop("access_token")

        refresh_token = d.pop("refresh_token")

        expires_in = d.pop("expires_in")

        token_type = d.pop("token_type")

        user = User.from_dict(d.pop("user"))

        def _parse_provider_token(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        provider_token = _parse_provider_token(d.pop("provider_token", UNSET))

        def _parse_provider_refresh_token(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        provider_refresh_token = _parse_provider_refresh_token(d.pop("provider_refresh_token", UNSET))

        def _parse_expires_at(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        expires_at = _parse_expires_at(d.pop("expires_at", UNSET))

        session = cls(
            access_token=access_token,
            refresh_token=refresh_token,
            expires_in=expires_in,
            token_type=token_type,
            user=user,
            provider_token=provider_token,
            provider_refresh_token=provider_refresh_token,
            expires_at=expires_at,
        )

        session.additional_properties = d
        return session

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0(str, Enum):
    ASC = "asc"
    DESC = "desc"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tool_resources_file_search_vector_store_chunking_strategy_auto import (
        ToolResourcesFileSearchVectorStoreChunkingStrategyAuto,
    )
    from ..models.tool_resources_file_search_vector_store_chunking_strategy_static import (
        ToolResourcesFileSearchVectorStoreChunkingStrategyStatic,
    )
    from ..models.tool_resources_file_search_vector_store_metadata_type_0 import (
        ToolResourcesFileSearchVectorStoreMetadataType0,
    )


T = TypeVar("T", bound="ToolResourcesFileSearchVectorStore")


@_attrs_define
class ToolResourcesFileSearchVectorStore:
    """
    Attributes:
        chunking_strategy (ToolResourcesFileSearchVectorStoreChunkingStrategyAuto |
            ToolResourcesFileSearchVectorStoreChunkingStrategyStatic | Unset):
        file_ids (list[str] | Unset):
        metadata (None | ToolResourcesFileSearchVectorStoreMetadataType0 | Unset):
    """

    chunking_strategy: (
        ToolResourcesFileSearchVectorStoreChunkingStrategyAuto
        | ToolResourcesFileSearchVectorStoreChunkingStrategyStatic
        | Unset
    ) = UNSET
    file_ids: list[str] | Unset = UNSET
    metadata: None | ToolResourcesFileSearchVectorStoreMetadataType0 | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.tool_resources_file_search_vector_store_chunking_strategy_auto import (
            ToolResourcesFileSearchVectorStoreChunkingStrategyAuto,
        )
        from ..models.tool_resources_file_search_vector_store_metadata_type_0 import (
            ToolResourcesFileSearchVectorStoreMetadataType0,
        )

        chunking_strategy: dict[str, Any] | Unset
        if isinstance(self.chunking_strategy, Unset):
            chunking_strategy = UNSET
        elif isinstance(self.chunking_strategy, ToolResourcesFileSearchVectorStoreChunkingStrategyAuto):
            chunking_strategy = self.chunking_strategy.to_dict()
        else:
            chunking_strategy = self.chunking_strategy.to_dict()

        file_ids: list[str] | Unset = UNSET
        if not isinstance(self.file_ids, Unset):
            file_ids = self.file_ids

        metadata: dict[str, Any] | None | Unset
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, ToolResourcesFileSearchVectorStoreMetadataType0):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if chunking_strategy is not UNSET:
            field_dict["chunking_strategy"] = chunking_strategy
        if file_ids is not UNSET:
            field_dict["file_ids"] = file_ids
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tool_resources_file_search_vector_store_chunking_strategy_auto import (
            ToolResourcesFileSearchVectorStoreChunkingStrategyAuto,
        )
        from ..models.tool_resources_file_search_vector_store_chunking_strategy_static import (
            ToolResourcesFileSearchVectorStoreChunkingStrategyStatic,
        )
        from ..models.tool_resources_file_search_vector_store_metadata_type_0 import (
            ToolResourcesFileSearchVectorStoreMetadataType0,
        )

        d = dict(src_dict)

        def _parse_chunking_strategy(
            data: object,
        ) -> (
            ToolResourcesFileSearchVectorStoreChunkingStrategyAuto
            | ToolResourcesFileSearchVectorStoreChunkingStrategyStatic
            | Unset
        ):
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                chunking_strategy_type_0 = ToolResourcesFileSearchVectorStoreChunkingStrategyAuto.from_dict(data)

                return chunking_strategy_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            chunking_strategy_type_1 = ToolResourcesFileSearchVectorStoreChunkingStrategyStatic.from_dict(data)

            return chunking_strategy_type_1

        chunking_strategy = _parse_chunking_strategy(d.pop("chunking_strategy", UNSET))

        file_ids = cast(list[str], d.pop("file_ids", UNSET))

        def _parse_metadata(data: object) -> None | ToolResourcesFileSearchVectorStoreMetadataType0 | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_type_0 = ToolResourcesFileSearchVectorStoreMetadataType0.from_dict(data)

                return metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ToolResourcesFileSearchVectorStoreMetadataType0 | Unset, data)

        metadata = _parse_metadata(d.pop("metadata", UNSET))

        tool_resources_file_search_vector_store = cls(
            chunking_strategy=chunking_strategy,
            file_ids=file_ids,
            metadata=metadata,
        )

        tool_resources_file_search_vector_store.additional_properties = d
        return tool_resources_file_search_vector_store

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="BodyProcessPasswordResetPasswordResetPost")


@_attrs_define
class BodyProcessPasswordResetPasswordResetPost:
    """
    Attributes:
        new_password (str):
        confirm_password (str):
    """

    new_password: str
    confirm_password: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        new_password = self.new_password

        confirm_password = self.confirm_password

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "new_password": new_password,
                "confirm_password": confirm_password,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        new_password = d.pop("new_password")

        confirm_password = d.pop("confirm_password")

        body_process_password_reset_password_reset_post = cls(
            new_password=new_password,
            confirm_password=confirm_password,
        )

        body_process_password_reset_password_reset_post.additional_properties = d
        return body_process_password_reset_password_reset_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateVectorStore")


@_attrs_define
class UpdateVectorStore:
    """
    Attributes:
        vector_store_id (str):
        name (None | str | Unset):
        after (None | str | Unset):
        before (None | str | Unset):
    """

    vector_store_id: str
    name: None | str | Unset = UNSET
    after: None | str | Unset = UNSET
    before: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        vector_store_id = self.vector_store_id

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        after: None | str | Unset
        if isinstance(self.after, Unset):
            after = UNSET
        else:
            after = self.after

        before: None | str | Unset
        if isinstance(self.before, Unset):
            before = UNSET
        else:
            before = self.before

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "vector_store_id": vector_store_id,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if after is not UNSET:
            field_dict["after"] = after
        if before is not UNSET:
            field_dict["before"] = before

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        vector_store_id = d.pop("vector_store_id")

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_after(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        after = _parse_after(d.pop("after", UNSET))

        def _parse_before(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        before = _parse_before(d.pop("before", UNSET))

        update_vector_store = cls(
            vector_store_id=vector_store_id,
            name=name,
            after=after,
            before=before,
        )

        update_vector_store.additional_properties = d
        return update_vector_store

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DataFlowType(str, Enum):
    AS_IS_LAST_OUTPUT = "as_is_last_output"
    BOTH_INPUT_OUTPUT = "both_input_output"
    ONLY_OUTPUT = "only_output"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.session import Session
    from ..models.user import User


T = TypeVar("T", bound="AuthResponse")


@_attrs_define
class AuthResponse:
    """
    Attributes:
        user (None | Unset | User):
        session (None | Session | Unset):
    """

    user: None | Unset | User = UNSET
    session: None | Session | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.session import Session
        from ..models.user import User

        user: dict[str, Any] | None | Unset
        if isinstance(self.user, Unset):
            user = UNSET
        elif isinstance(self.user, User):
            user = self.user.to_dict()
        else:
            user = self.user

        session: dict[str, Any] | None | Unset
        if isinstance(self.session, Unset):
            session = UNSET
        elif isinstance(self.session, Session):
            session = self.session.to_dict()
        else:
            session = self.session

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if session is not UNSET:
            field_dict["session"] = session

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.session import Session
        from ..models.user import User

        d = dict(src_dict)

        def _parse_user(data: object) -> None | Unset | User:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                user_type_0 = User.from_dict(data)

                return user_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | User, data)

        user = _parse_user(d.pop("user", UNSET))

        def _parse_session(data: object) -> None | Session | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                session_type_0 = Session.from_dict(data)

                return session_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Session | Unset, data)

        session = _parse_session(d.pop("session", UNSET))

        auth_response = cls(
            user=user,
            session=session,
        )

        auth_response.additional_properties = d
        return auth_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.chat_completion_allowed_tools_param import ChatCompletionAllowedToolsParam


T = TypeVar("T", bound="ChatCompletionAllowedToolChoiceParam")


@_attrs_define
class ChatCompletionAllowedToolChoiceParam:
    """Constrains the tools available to the model to a pre-defined set.

    Attributes:
        allowed_tools (ChatCompletionAllowedToolsParam): Constrains the tools available to the model to a pre-defined
            set.
        type_ (Literal['allowed_tools']):
    """

    allowed_tools: ChatCompletionAllowedToolsParam
    type_: Literal["allowed_tools"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        allowed_tools = self.allowed_tools.to_dict()

        type_ = self.type_

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "allowed_tools": allowed_tools,
                "type": type_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.chat_completion_allowed_tools_param import ChatCompletionAllowedToolsParam

        d = dict(src_dict)
        allowed_tools = ChatCompletionAllowedToolsParam.from_dict(d.pop("allowed_tools"))

        type_ = cast(Literal["allowed_tools"], d.pop("type"))
        if type_ != "allowed_tools":
            raise ValueError(f"type must match const 'allowed_tools', got '{type_}'")

        chat_completion_allowed_tool_choice_param = cls(
            allowed_tools=allowed_tools,
            type_=type_,
        )

        chat_completion_allowed_tool_choice_param.additional_properties = d
        return chat_completion_allowed_tool_choice_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.auto_file_chunking_strategy_param import AutoFileChunkingStrategyParam
    from ..models.create_vector_store_metadata_type_0 import CreateVectorStoreMetadataType0
    from ..models.expires_after import ExpiresAfter
    from ..models.static_file_chunking_strategy_object_param import StaticFileChunkingStrategyObjectParam


T = TypeVar("T", bound="CreateVectorStore")


@_attrs_define
class CreateVectorStore:
    """
    Attributes:
        chunking_strategy (AutoFileChunkingStrategyParam | None | StaticFileChunkingStrategyObjectParam | Unset):
        expires_after (ExpiresAfter | None | Unset):
        file_ids (list[str] | None | Unset):
        metadata (CreateVectorStoreMetadataType0 | None | Unset):
        name (None | str | Unset):
    """

    chunking_strategy: AutoFileChunkingStrategyParam | None | StaticFileChunkingStrategyObjectParam | Unset = UNSET
    expires_after: ExpiresAfter | None | Unset = UNSET
    file_ids: list[str] | None | Unset = UNSET
    metadata: CreateVectorStoreMetadataType0 | None | Unset = UNSET
    name: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.auto_file_chunking_strategy_param import AutoFileChunkingStrategyParam
        from ..models.create_vector_store_metadata_type_0 import CreateVectorStoreMetadataType0
        from ..models.expires_after import ExpiresAfter
        from ..models.static_file_chunking_strategy_object_param import StaticFileChunkingStrategyObjectParam

        chunking_strategy: dict[str, Any] | None | Unset
        if isinstance(self.chunking_strategy, Unset):
            chunking_strategy = UNSET
        elif isinstance(self.chunking_strategy, AutoFileChunkingStrategyParam):
            chunking_strategy = self.chunking_strategy.to_dict()
        elif isinstance(self.chunking_strategy, StaticFileChunkingStrategyObjectParam):
            chunking_strategy = self.chunking_strategy.to_dict()
        else:
            chunking_strategy = self.chunking_strategy

        expires_after: dict[str, Any] | None | Unset
        if isinstance(self.expires_after, Unset):
            expires_after = UNSET
        elif isinstance(self.expires_after, ExpiresAfter):
            expires_after = self.expires_after.to_dict()
        else:
            expires_after = self.expires_after

        file_ids: list[str] | None | Unset
        if isinstance(self.file_ids, Unset):
            file_ids = UNSET
        elif isinstance(self.file_ids, list):
            file_ids = self.file_ids

        else:
            file_ids = self.file_ids

        metadata: dict[str, Any] | None | Unset
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, CreateVectorStoreMetadataType0):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if chunking_strategy is not UNSET:
            field_dict["chunking_strategy"] = chunking_strategy
        if expires_after is not UNSET:
            field_dict["expires_after"] = expires_after
        if file_ids is not UNSET:
            field_dict["file_ids"] = file_ids
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.auto_file_chunking_strategy_param import AutoFileChunkingStrategyParam
        from ..models.create_vector_store_metadata_type_0 import CreateVectorStoreMetadataType0
        from ..models.expires_after import ExpiresAfter
        from ..models.static_file_chunking_strategy_object_param import StaticFileChunkingStrategyObjectParam

        d = dict(src_dict)

        def _parse_chunking_strategy(
            data: object,
        ) -> AutoFileChunkingStrategyParam | None | StaticFileChunkingStrategyObjectParam | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                chunking_strategy_type_0 = AutoFileChunkingStrategyParam.from_dict(data)

                return chunking_strategy_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                chunking_strategy_type_1 = StaticFileChunkingStrategyObjectParam.from_dict(data)

                return chunking_strategy_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AutoFileChunkingStrategyParam | None | StaticFileChunkingStrategyObjectParam | Unset, data)

        chunking_strategy = _parse_chunking_strategy(d.pop("chunking_strategy", UNSET))

        def _parse_expires_after(data: object) -> ExpiresAfter | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                expires_after_type_0 = ExpiresAfter.from_dict(data)

                return expires_after_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ExpiresAfter | None | Unset, data)

        expires_after = _parse_expires_after(d.pop("expires_after", UNSET))

        def _parse_file_ids(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                file_ids_type_0 = cast(list[str], data)

                return file_ids_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        file_ids = _parse_file_ids(d.pop("file_ids", UNSET))

        def _parse_metadata(data: object) -> CreateVectorStoreMetadataType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_type_0 = CreateVectorStoreMetadataType0.from_dict(data)

                return metadata_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(CreateVectorStoreMetadataType0 | None | Unset, data)

        metadata = _parse_metadata(d.pop("metadata", UNSET))

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        create_vector_store = cls(
            chunking_strategy=chunking_strategy,
            expires_after=expires_after,
            file_ids=file_ids,
            metadata=metadata,
            name=name,
        )

        create_vector_store.additional_properties = d
        return create_vector_store

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.list_vector_stores_v1_openai_storage_vector_stores_list_get_openai_provider import (
    ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider,
)
from ...models.list_vector_stores_v1_openai_storage_vector_stores_list_get_order import (
    ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder,
)
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    openai_provider: ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider,
    limit: int | Unset = 10,
    order: ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder
    | Unset = ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_openai_provider = openai_provider.value
    params["openai_provider"] = json_openai_provider

    params["limit"] = limit

    json_order: str | Unset = UNSET
    if not isinstance(order, Unset):
        json_order = order.value

    params["order"] = json_order

    json_after: None | str | Unset
    if isinstance(after, Unset):
        json_after = UNSET
    else:
        json_after = after
    params["after"] = json_after

    json_before: None | str | Unset
    if isinstance(before, Unset):
        json_before = UNSET
    else:
        json_before = before
    params["before"] = json_before

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/openai_storage/vector_stores/list",
        "params": params,
    }

    return _kwargs


def _parse_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> HTTPValidationError | None:
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: AuthenticatedClient | Client, response: httpx.Response) -> Response[HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider,
    limit: int | Unset = 10,
    order: ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder
    | Unset = ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
) -> Response[HTTPValidationError]:
    """List Vector Stores

    Args:
        openai_provider (ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider):
        limit (int | Unset):  Default: 10.
        order (ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder | Unset):  Default:
            ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC.
        after (None | str | Unset):
        before (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError]
    """

    kwargs = _get_kwargs(
        openai_provider=openai_provider,
        limit=limit,
        order=order,
        after=after,
        before=before,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider,
    limit: int | Unset = 10,
    order: ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder
    | Unset = ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
) -> HTTPValidationError | None:
    """List Vector Stores

    Args:
        openai_provider (ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider):
        limit (int | Unset):  Default: 10.
        order (ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder | Unset):  Default:
            ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC.
        after (None | str | Unset):
        before (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError
    """

    return sync_detailed(
        client=client,
        openai_provider=openai_provider,
        limit=limit,
        order=order,
        after=after,
        before=before,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider,
    limit: int | Unset = 10,
    order: ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder
    | Unset = ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
) -> Response[HTTPValidationError]:
    """List Vector Stores

    Args:
        openai_provider (ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider):
        limit (int | Unset):  Default: 10.
        order (ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder | Unset):  Default:
            ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC.
        after (None | str | Unset):
        before (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError]
    """

    kwargs = _get_kwargs(
        openai_provider=openai_provider,
        limit=limit,
        order=order,
        after=after,
        before=before,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider,
    limit: int | Unset = 10,
    order: ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder
    | Unset = ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
) -> HTTPValidationError | None:
    """List Vector Stores

    Args:
        openai_provider (ListVectorStoresV1OpenaiStorageVectorStoresListGetOpenaiProvider):
        limit (int | Unset):  Default: 10.
        order (ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder | Unset):  Default:
            ListVectorStoresV1OpenaiStorageVectorStoresListGetOrder.DESC.
        after (None | str | Unset):
        before (None | str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            openai_provider=openai_provider,
            limit=limit,
            order=order,
            after=after,
            before=before,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from .. import types

T = TypeVar("T", bound="BodyUploadFileOnOpenaiV1OpenaiStorageFilesOpenaiPost")


@_attrs_define
class BodyUploadFileOnOpenaiV1OpenaiStorageFilesOpenaiPost:
    """
    Attributes:
        file (str): File as UploadFile
    """

    file: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        file = self.file

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "file": file,
            }
        )

        return field_dict

    def to_multipart(self) -> types.RequestFiles:
        files: types.RequestFiles = []

        files.append(("file", (None, str(self.file).encode(), "text/plain")))

        for prop_name, prop in self.additional_properties.items():
            files.append((prop_name, (None, str(prop).encode(), "text/plain")))

        return files

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        file = d.pop("file")

        body_upload_file_on_openai_v1_openai_storage_files_openai_post = cls(
            file=file,
        )

        body_upload_file_on_openai_v1_openai_storage_files_openai_post.additional_properties = d
        return body_upload_file_on_openai_v1_openai_storage_files_openai_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="FileFile")


@_attrs_define
class FileFile:
    """
    Attributes:
        file_data (str | Unset):
        file_id (str | Unset):
        filename (str | Unset):
    """

    file_data: str | Unset = UNSET
    file_id: str | Unset = UNSET
    filename: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        file_data = self.file_data

        file_id = self.file_id

        filename = self.filename

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if file_data is not UNSET:
            field_dict["file_data"] = file_data
        if file_id is not UNSET:
            field_dict["file_id"] = file_id
        if filename is not UNSET:
            field_dict["filename"] = filename

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        file_data = d.pop("file_data", UNSET)

        file_id = d.pop("file_id", UNSET)

        filename = d.pop("filename", UNSET)

        file_file = cls(
            file_data=file_data,
            file_id=file_id,
            filename=filename,
        )

        file_file.additional_properties = d
        return file_file

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
